#ifndef __FTPLOTLIB_H_INCLUDED
#define __FTPLOTLIB_H_INCLUDED

#include "umfish30VC.h"

class ftPlotter
{
private:
    union ZTabStruct    // Struktur der Zeichentabelle
    {
        struct 
        {
            byte y:4;
            byte x:3;
            byte pen:1;
        };
        byte b;
    };

private:
    long XM, YM;        // --- Breite, Hhe der Plotterflche in Plotterschritten
    long XJ, YJ;        // --- Absolute Koordinaten des akt. Punktes
                        //     (0,0)   : linke, untere Ecke
                        //     Einheit : ein Plotterschritt
    double XH, YH;      // --- Virtueller (logischer) Nullpunkt
                        //     in absoluten Koordinaten
    double XS, YS;      // --- Skalierung : Anzahl Plotterschritte/log Einheit
    bool Xout, Yout;    // --- True wenn XJ/YJ auerhalb
                        //     der Plotterflche 0 ... XM, 0 ... YM
    bool MG;            // --- Status Stift

    static byte ZTab[256][20];   // --- Zeichentabelle
    ftiDCB ft;          // --- DCB zur Plottersteuerung

public:
    // initialization
    ftPlotter(CString interf = "COM1");
    void Home();
private:
    void InitZTab();
    void StartFtPlot(CString interf);
    void StopFtPlot();
    void ClearMotors();
    void Beenden();

public:
    // basic drawing functions
    void Origin();
    void Scale(double XMin, double XMax, double YMin, double YMax);
    void SetOrigin(int X, int Y);

    void Move(double X, double Y);
    void Line(double X, double Y);
    void RMove(double X, double Y);
    void RLine(double X, double Y);
    void Box(double XA, double XE, double YA, double YE, int S = 0);
    void Circle(double X, double Y, double RX, double RY, double WA, double WE);

    // character drawing functions
    void Char(double X, double Y, CString A, int R, int G);
    void RChar(double X, double Y, CString A, int R, int G);
    void Char(CString A, int R, int G);

    // hight level drawing functions
    void Axis(double XA, double YA, double XE, double YE, double X1, double Y1, double X2, double Y2, 
                bool SK, CString XText, CString YText);

    // information functions
    inline long GetXMax() { return XM; };
    inline long GetYMax() { return YM; };

private:
    // basic funcions
    void XPlusSchritt();
    void YPlusSchritt();
    void XMinusSchritt();
    void YMinusSchritt();
    void XpYpDiagonale();
    void XpYmDiagonale();
    void XmYpDiagonale();
    void XmYmDiagonale();

    void Strich(int XR, int YR);
    void BereichsLimit();
    void EndTaster();

    // pen funtions
    void StiftAuf();
    void StiftAb();

    // internal character drawing functions
    void ZeichenFolge(CString A, int R, int G);
    void ZeichenVektor(int Xalt, int Xneu, int Yalt, int Yneu, bool Pens, int R, int G);

    // helper functions
    int sgn(double val);
    double abs_d(double val);
    int round(double val);
    int basic_int(double val);
};

#endif